/* REminiscence - Flashback interpreter
 * Copyright (C) 2005 Gregory Montoir
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "sys.h"

#include <time.h> 

#include "scaler.h"
#include "systemstub.h"


#include "vfs_external.h"



typedef struct rect { int x,y,w,h; } rect;

#ifdef GP2X


void (*callback_this)(void *param, uint8 *stream, int len);
void *callback_param;

void gp2x_sound_frame(void *blah, void *bufferg, int samples)
{
  uint8 buffert[2048*4], *buffer8=buffert;
  signed short *buffer16=(signed short *)bufferg;

  callback_this(callback_param, buffert, samples); //this callback needs numbytes, instead of numsamples

  while(samples--)
  {
   *buffer16++=((*buffer8))<<8; //signed mono 8 to signed stereo 16
   *buffer16++=((*buffer8))<<8;
   buffer8++;
  }
}

         void *vfs_dat_backup;
          int GP2X_SCALED=1;

void gp2x_cache_flashback_quit(void)
{
 free(vfs_dat_backup);
}

void gp2x_cache_flashback(void)
{
 unsigned char *vfs_datt;
 FILE *in;
 long t,k;
 unsigned short *screen16 = (unsigned short *)gp2x_video_RGB[0].screen;

 vfs_datt=(unsigned char*)(vfs_dat_backup=malloc(8*1024*1024));

 vfs_import(vfs_dat_backup);

 in=fopen("rs.dat","rb");
 if(in==NULL) exit(1);
 fseek(in, 0L, SEEK_END);
 k=t=ftell(in);
 fseek(in, 0L, SEEK_SET);

 while(t)
 {
  int percent=((k-t)*100)/k,y,x;

  y=118;
  for(x=160-50;x<160-50+100;x++)
    screen16[x+y*320]=0xFFFF;

  for(;y<=122;y++)
   for(x=160-50;x<160-50+1+percent;x++)
    screen16[x+y*320]=0xFFFF;

  for(x=160-50;x<160-50+100;x++)
    screen16[x+y*320]=0xFFFF;

  for(;y>=118;y--)
    screen16[x+y*320]=0xFFFF;

  gp2x_printf(NULL,(320-33*6)/2,118-3-10,"REminiscence v0.4 ported by rlyeh"); 
  gp2x_printf(NULL,(320-4*6)/2,122+5,"%03d%%\b",percent+1); 

  gp2x_video_RGB_flip(0);
  screen16 = (unsigned short *)gp2x_video_RGB[0].screen;

  #define BLOCK 512*1024
  if(t>BLOCK)
   { fread(vfs_datt, 1, BLOCK, in); t-=BLOCK; vfs_datt+=BLOCK; }
  else
   { fread(vfs_datt, 1, t, in); t-=t; }
 }

 fclose(in);

 atexit(gp2x_cache_flashback_quit);
}


void sys_preinit(void)
{
                printf("REminiscence 0.1.9 by Gregory Montoir (cyx), 2005-2007.\n" \
                       "REminiscence GP2X version v0.4 by rlyeh, 2005-2007. (" __DATE__ " " __TIME__ ")\n");

                gp2x_init(1000, 16, 44100,16,1,60, 1);
                gp2x_video_setgammacorrection(1.20);
                //gp2x_video_logo_enable(1);

		gp2x_dualcore_clock(100);

                #ifdef USE_VFS
                gp2x_cache_flashback();
                #endif

 memset(gp2x_video_RGB[0].screen,0,320*240*2); gp2x_video_RGB_flip(0);
 memset(gp2x_video_RGB[0].screen,0,320*240*2); gp2x_video_RGB_flip(0);
 memset(gp2x_video_RGB[0].screen,0,320*240*2); gp2x_video_RGB_flip(0);

                gp2x_video_RGB_setscaling(256,224);
}

#endif



struct SystemStub_SDL : SystemStub {
	enum {
		MAX_BLIT_RECTS = 200,
#ifdef GP2X
		SOUND_SAMPLE_RATE = 44100, 
#else
		SOUND_SAMPLE_RATE = 11025, 
#endif
	};

	uint8 *_offscreen;

        unsigned short *_screen;

	uint8 _scaler;
	uint8 _overscanColor;
	uint16 _pal[256];
	uint16 _screenW, _screenH;
        rect   _blitRects[MAX_BLIT_RECTS];
	uint16 _numBlitRects;
	
	virtual ~SystemStub_SDL() {}
	virtual void init(const char *title, uint16 w, uint16 h);
	virtual void destroy();
	virtual void setPalette(const uint8 *pal, uint16 n);
	virtual void setPaletteEntry(uint8 i, const Color *c);
	virtual void getPaletteEntry(uint8 i, Color *c);
	virtual void setOverscanColor(uint8 i);
	virtual void copyRect(int16 x, int16 y, uint16 w, uint16 h, const uint8 *buf, uint32 pitch);
	virtual void updateScreen(uint8 shakeOffset);
	virtual void processEvents();
	virtual void sleep(uint32 duration);
	virtual uint32 getTimeStamp();
	virtual void startAudio(AudioCallback callback, void *param);
	virtual void stopAudio();
	virtual uint32 getOutputSampleRate();
	virtual void *createMutex();
	virtual void destroyMutex(void *mutex);
	virtual void lockMutex(void *mutex);
	virtual void unlockMutex(void *mutex);

	void prepareGfxMode();
	void cleanupGfxMode();
	void switchGfxMode(bool fullscreen, uint8 scaler);
        void drawRect(rect *rct, uint8 color, uint16 *dst, uint16 dstPitch);
};

SystemStub *SystemStub_SDL_create() {
	return new SystemStub_SDL();
}

void SystemStub_SDL::init(const char *title, uint16 w, uint16 h) {

	memset(&_pi, 0, sizeof(_pi));
	_screenW = w;
	_screenH = h;

	// allocate some extra bytes for the scaling routines
	int size_offscreen = (w + 2) * (h + 2) * 2;
        _offscreen = (uint8 *)malloc(size_offscreen);
	if (!_offscreen) {
		error("SystemStub_SDL::init() Unable to allocate offscreen buffer");
	}
	memset(_offscreen, 0, size_offscreen);

	_scaler = 0;

	memset(_pal, 0, sizeof(_pal));
	prepareGfxMode();
}


void SystemStub_SDL::setPalette(const uint8 *pal, uint16 n) {
	assert(n <= 256);
	for (int i = 0; i < n; ++i) {
		uint8 r = pal[i * 3 + 0];
		uint8 g = pal[i * 3 + 1];
		uint8 b = pal[i * 3 + 2];
                _pal[i] = gp2x_video_RGB_color15(r,g,b,0);
	}	
}

void SystemStub_SDL::setPaletteEntry(uint8 i, const Color *c) {
	uint8 r = (c->r << 2) | (c->r & 3);
	uint8 g = (c->g << 2) | (c->g & 3);
	uint8 b = (c->b << 2) | (c->b & 3);
        _pal[i] = gp2x_video_RGB_color15(r,g,b,0);
}

void SystemStub_SDL::getPaletteEntry(uint8 i, Color *c) {
        c->r = (_pal[i] & gp2x_video_RGB_color15(255,0,0,0)) / gp2x_video_RGB_color15(255,0,0,0);
        c->g = (_pal[i] & gp2x_video_RGB_color15(0,255,0,0)) / gp2x_video_RGB_color15(0,255,0,0);
        c->b = (_pal[i] & gp2x_video_RGB_color15(0,0,255,0)) / gp2x_video_RGB_color15(0,0,255,0);

        c->r >>= 2;
	c->g >>= 2;
	c->b >>= 2;
}


void SystemStub_SDL::updateScreen(uint8 shakeOffset) {
	const int mul = _scalers[_scaler].factor;
	if (shakeOffset == 0) {
		for (int i = 0; i < _numBlitRects; ++i) {
                        rect *br = &_blitRects[i];
			int16 dx = br->x * mul;
			int16 dy = br->y * mul;
                        uint16 *dst = (uint16 *)_screen + dy * _screenW*2 / 2 + dx;
			const uint16 *src = (uint16 *)_offscreen + (br->y + 1) * _screenW + (br->x + 1);
                        (*_scalers[_scaler].proc)(dst, _screenW*2, src, _screenW, br->w, br->h);
		}
	} else {
                memset(_screen, _pal[_overscanColor], _screenW*2 * shakeOffset);

		uint16 w = _screenW;
		uint16 h = _screenH - shakeOffset;
                uint16 *dst = (uint16 *)_screen;
		const uint16 *src = (uint16 *)_offscreen + _screenW + 1;
                (*_scalers[_scaler].proc)(dst, _screenW*2, src, _screenW, w, h);
	}
	_numBlitRects = 0;

#ifdef GP2X
                int y=0;
                short *x=(short *)_screen, *screen16=(short *)gp2x_video_RGB[0].screen;

                if(GP2X_SCALED)

                for(y=0;y<224;y++)
                 memcpy(&screen16[ 0+(y+0)*320],&x[y*256],256*2);

                else

                for(y=0;y<224;y++)
                 memcpy(&screen16[32+(y+8)*320],&x[y*256],256*2);


                gp2x_video_RGB_flip(0);
#endif

}


void SystemStub_SDL::processEvents() {
        unsigned long joy=gp2x_joystick_read(0);

        _pi.dirMask=_pi.dbgMask=0;

        if(joy & GP2X_UP)    _pi.dirMask |= PlayerInput::DIR_UP;
        if(joy & GP2X_DOWN)  _pi.dirMask |= PlayerInput::DIR_DOWN;
        if(joy & GP2X_LEFT)  _pi.dirMask |= PlayerInput::DIR_LEFT,  _pi.dirMask &= ~PlayerInput::DIR_DOWN; 
        if(joy & GP2X_RIGHT) _pi.dirMask |= PlayerInput::DIR_RIGHT, _pi.dirMask &= ~PlayerInput::DIR_DOWN;

//      if(joy & GP2X_START) joy|=GP2X_Y|GP2X_A;

        _pi.quit=false;
        _pi.space=_pi.shift=_pi.enter=_pi.backspace=false;

        if(joy & GP2X_X)     _pi.space = true;
        if(joy & GP2X_B)     _pi.shift = true;
        if(joy & GP2X_R)     _pi.enter = true;
        if(joy & GP2X_A)     _pi.enter = true;

        if(joy & GP2X_SELECT)
        {
          if(GP2X_SCALED^=1) gp2x_video_RGB_setscaling(256,224);
           else gp2x_video_RGB_setscaling(320,240);

          memset(gp2x_video_RGB[0].screen,0,320*240*2); gp2x_video_RGB_flip(0);
          memset(gp2x_video_RGB[0].screen,0,320*240*2); gp2x_video_RGB_flip(0);
          memset(gp2x_video_RGB[0].screen,0,320*240*2); gp2x_video_RGB_flip(0);

          while(gp2x_joystick_read(0) & GP2X_SELECT);
        }

        if(joy & GP2X_START)
        {
         static int alpha=0;

         if(joy & GP2X_VOL_DOWN) exit(1);

         //gp2x_video_setdithering(alpha^=1);

         //alpha^=0x80;
         //gp2x_video_setluminance(3,alpha+0x7F);
         //while(gp2x_joystick_read(0) & GP2X_START);
        }


        {
         static int vol=100;

         if(joy & GP2X_VOL_DOWN) if(vol>0)   { vol--; gp2x_sound_volume(vol,vol); }
         if(joy & GP2X_VOL_UP)   if(vol<100) { vol++; gp2x_sound_volume(vol,vol); }
        }

	/*
        {
         static int flag=0;
         if(!flag) { if(  joy & GP2X_A || joy & GP2X_R)      flag=1, _pi.enter = true; }
          else     { if(!(joy & GP2X_A) && !(joy & GP2X_R))  flag=0;  }
        }
	*/

        
        {
         static int flag=0;
         if(!flag) { if(joy & GP2X_Y)    flag=1, _pi.backspace = true; }
          else     { if(!(joy & GP2X_Y)) flag=0;  }
        }
        




        /*
					_pi.save = true;
					_pi.load = true;
					_pi.stateSlot = 1;
                                        _pi.stateSlot = -1;
        */
}


  void SystemStub_SDL::setOverscanColor(uint8 i)        { _overscanColor = i; }
  void SystemStub_SDL::destroy()                        { cleanupGfxMode(); }
  void SystemStub_SDL::sleep(uint32 duration)           { gp2x_timer_delay(duration); }
uint32 SystemStub_SDL::getTimeStamp()                   { return gp2x_timer_read(); }
uint32 SystemStub_SDL::getOutputSampleRate()            { return SOUND_SAMPLE_RATE; }
 void *SystemStub_SDL::createMutex()                    { return NULL; }
 void  SystemStub_SDL::destroyMutex(void *mutex)        {}
 void  SystemStub_SDL::lockMutex(void *mutex)           {}
 void  SystemStub_SDL::unlockMutex(void *mutex)         {}
  void SystemStub_SDL::switchGfxMode(bool fullscreen, uint8 scaler) {}
  void SystemStub_SDL::startAudio(AudioCallback callback, void *param) { callback_this=callback; callback_param=param; gp2x_sound_pause(0); }
  void SystemStub_SDL::stopAudio()                      { callback_this=NULL; }


void SystemStub_SDL::prepareGfxMode() {
      int w = _screenW * _scalers[_scaler].factor;
      int h = _screenH * _scalers[_scaler].factor;

      _screen = (unsigned short *)malloc(w*h*2);
      if (!_screen) {
		error("SystemStub_SDL::prepareGfxMode() unable to allocate _screen buffer");
      }

        _numBlitRects = 1;
        _blitRects[0].x = 0;
        _blitRects[0].y = 0;
        _blitRects[0].w = _screenW;
        _blitRects[0].h = _screenH;
}

void SystemStub_SDL::cleanupGfxMode() {
	if (_offscreen) {
		free(_offscreen);
		_offscreen = 0;
	}
	if (_screen) {
                free(_screen);
                _screen = 0;
	}
}




void SystemStub_SDL::drawRect(rect *rct, uint8 color, uint16 *dst, uint16 dstPitch) {
	dstPitch >>= 1;
        int x1 = rct->x;
        int y1 = rct->y;
        int x2 = rct->x + rct->w - 1;
        int y2 = rct->y + rct->h - 1;
	assert(x1 >= 0 && x2 < _screenW && y1 >= 0 && y2 < _screenH);
	for (int i = x1; i <= x2; ++i) {
		*(dst + y1 * dstPitch + i) = *(dst + y2 * dstPitch + i) = _pal[color];
	}
	for (int j = y1; j <= y2; ++j) {
		*(dst + j * dstPitch + x1) = *(dst + j * dstPitch + x2) = _pal[color];
	}
}

void SystemStub_SDL::copyRect(int16 x, int16 y, uint16 w, uint16 h, const uint8 *buf, uint32 pitch) {
	if (_numBlitRects >= MAX_BLIT_RECTS) {
		warning("SystemStub_SDL::copyRect() Too many blit rects, you may experience graphical glitches");
	} else {		
		// extend the dirty region by 1 pixel for scalers accessing 'outer' pixels
		--x;
		--y;
		w += 2;
		h += 2;
		
		if (x < 0) {
			x = 0;
		}
		if (y < 0) {
			y = 0;
		}
		if (x + w > _screenW) {
			w = _screenW - x;
		}
		if (y + h > _screenH) {
			h = _screenH - y;
		}

                rect *br = &_blitRects[_numBlitRects];
		
		br->x = x;
		br->y = y;
		br->w = w;
		br->h = h;
		++_numBlitRects;
		
		uint16 *p = (uint16 *)_offscreen + (y + 1) * _screenW + (x + 1);
		buf += y * pitch + x;
		while (h--) {
			for (int i = 0; i < w; ++i) {
				p[i] = _pal[buf[i]];
			}
			p += _screenW;
			buf += pitch;
		}
		if (_pi.dbgMask & PlayerInput::DF_DBLOCKS) {
			drawRect(br, 0xE7, (uint16 *)_offscreen + _screenW + 1, _screenW * 2);
		}
	}
}
